import { Body, Controller, Get, Put, Query, UseGuards } from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionService } from './institucion.service';
import { IdInstitucionDto } from '../dto/id-institucion.dto';
import { UpdateInstitucionDto } from './dto/input/update.dto';
import { InstitucionOutputDto } from './dto/output/institucion.dto';
import { InstitucionMinOutputDto } from './dto/output/institucion-min.dto';
import { InstitucionesOutputDto } from './dto/output/instituciones.dto';
import { InstitucionesMinOutputDto } from './dto/output/instituciones-min.dto';

@Controller('institucion')
@ApiTags('institucion')
export class InstitucionController {
  constructor(private institucionService: InstitucionService) {}

  @Serealize(InstitucionMinOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que retorna todas las instituciones.',
  })
  @ApiBearerAuth('jwt')
  get() {
    return this.institucionService.findAll();
  }

  @Serealize(InstitucionesOutputDto)
  @Get('instituciones')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que retorna todas las instituciones.',
  })
  @ApiBearerAuth('jwt')
  instituciones() {
    return this.institucionService.findAll();
  }

  @Serealize(InstitucionOutputDto)
  @Get('institucion')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que retorna la información de una institución.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  institucion(@Query() query: IdInstitucionDto) {
    return this.institucionService.findById(parseInt(query.id_institucion));
  }

  @Serealize(InstitucionesMinOutputDto)
  @Get('activas')
  @ApiOperation({
    description: 'Endpoint que retorna todas las instituciones activas.',
  })
  activas() {
    return this.institucionService.findAll(true);
  }

  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que actualiza la información de una institución.',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description:
      'Todas las variables a excepción de id_institucion son opcionales.',
    examples: {
      ejemplo: {
        value: {
          id_institucion: 200,
          _activo: true,
          _dias_multa_retraso: 7,
          _dominio: '@pcpuma.acatlan.unam.mx',
          _email_institucional: true,
          _responsable: '',
          _telefono: '',
          _tiempo_entrega: 15,
          _tiempo_prestamo: 120,
          _tiempo_recoger: 10,
          _ubicacion: '',
        },
      },
    },
  })
  update(@Body() body: UpdateInstitucionDto) {
    return this.institucionService.update(body);
  }
}
