import { Body, Controller, Get, Post, Query, UseGuards } from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { ModeloService } from './modelo.service';
import { CreateModeloDto } from './dto/input/create.dto';
import { ModeloDto } from './dto/input/modelo.dto';
import { ModeloOutputDto } from './dto/output/modelo.dto';

@Controller('modelo')
@ApiTags('modelo')
export class ModeloController {
  constructor(private modeloService: ModeloService) {}

  @Serealize(ModeloOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Endpoint que retorna todos los modelos.' })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Tipo de marca que se busca.',
    name: 'tipo',
    type: 'text',
  })
  get(@Query() query: ModeloDto) {
    return this.modeloService.findAll(query.tipo);
  }

  @Post()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Endpoint que crea un nuevo modelo.' })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Todas las variables son obligatorias.',
    examples: { ejemplo: { value: { modelo: '', tipo: '' } } },
  })
  create(@Body() body: CreateModeloDto) {
    return this.modeloService.create(body.modelo, body.tipo);
  }
}
